#!/usr/bin/perl -w

use strict;
use Test::More;

#sub POE::Kernel::TRACE_EVENTS {1}
sub POE::Kernel::ASSERT_EVENTS {1}

use LWP::UserAgent;
use HTTP::Request;
use POE::Kernel;
use POE::Component::Server::HTTP;
use IO::Socket::INET;

sub DEBUG () { 0 };
my $PORT=2080;

my $pid=fork;
die "Unable to fork: $!" unless defined $pid;

END {
    if($pid) {
        kill 2, $pid or ($!==3) or warn "Unable to kill $pid: $!";
    }
}

####################################################################
unless($pid) {                      # we are child
    Test::Builder->no_ending(1);
    # stop kernel from griping
    ${$poe_kernel->[POE::Kernel::KR_RUN]} |=
                                POE::Kernel::KR_RUN_CALLED;

    print STDERR "$$: Sleep 2...";
    sleep 2;
    print STDERR "continue\n";


    ############################
    # 1, 2, 3
    my $sock=IO::Socket::INET->new(PeerAddr=>'localhost',
                                   PeerPort=>$PORT);
    $sock or die "Unable to connect to localhost:$PORT: $!";
    $sock->close;       # taunt other end

    ############################
    # 4, 5, 6
    $sock=IO::Socket::INET->new(PeerAddr=>'localhost',
                                   PeerPort=>$PORT);
    $sock or die "Unable to connect to localhost:$PORT: $!";

    my $req=HTTP::Request->new(GET => "http://localhost:$PORT/");
    $sock->print(join ' ', $req->method, $req->uri->as_string, "0\n");
    sleep 1;
    $sock->close;       # taunt other end

    ############################
    # 7, 8, 9
    $sock=IO::Socket::INET->new(PeerAddr=>'localhost',
                                   PeerPort=>$PORT);
    $sock or die "Unable to connect to localhost:$PORT: $!";
    $req=HTTP::Request->new(GET => "http://localhost:$PORT/honk");
    $sock->print($req->as_string);
    $sock->close;       # taunt other end

    ############################
    # 10, 11
    $req=HTTP::Request->new(GET => "http://localhost:$PORT/honk/shutdown.html");
    my $UA = LWP::UserAgent->new;
    my $resp=$UA->request($req);
    DEBUG and warn "done";
    exit 0;
}

####################################################################
# we are the parent

plan tests=>10;

my $Q=1;
my $shutdown=0;
my $top=0;
my $bonk=0;

my $aliases = POE::Component::Server::HTTP->new(
     Port => $PORT,
     Address=>'localhost',
     ContentHandler => { '/' => \&top,
                         '/honk/shutdown.html' => \&shutdown,
                         '/bonk/' => \&bonk
                         },
     PostHandler => {
            '/' => \&post_top,
            '/honk/shutdown.html' => \&post_shutdown,
     },
     ErrorHandler => { '/' => \&error },
     Headers => { Server => 'TestServer' },
  );

POE::Session->create(
        inline_states => {
           _start => sub {
                $poe_kernel->alias_set('HONK');
            },
        });


$poe_kernel->run;


#######################################
sub error
{
    my ($request, $response) = @_;

    DEBUG and __peek(1);

    die "Why is Q=$Q" unless $Q;

    ok(($request->is_error and $response->is_error), "this is an error");
    my $op=$request->header('Operation');
    my $errstr=$request->header('Error');
    my $errnum=$request->header('Errnum');

    DEBUG and
        warn "$$: ERROR op=$op errnum=$errnum errstr=$errstr\n";

    if($Q <= 3) {
        ok(($op eq 'read' and $errnum == 0), "closed connection") or
            die "Why did i get this error? op=$op errnum=$errnum errstr=$errstr";
    }
    else {
        die "Whoah!";
    }

    $Q++;
    return RC_OK;
}

#######################################
sub top
{
    my ($request, $response) = @_;
    $response->code(RC_OK);
    $response->content_type('text/plain');
    $response->content("this is top");
    $top=1;
    return RC_OK;
}

#######################################
sub bonk
{
    my ($request, $response) = @_;
    $response->code(RC_OK);
    $response->content_type('text/plain');
    $response->content("this is bonk");
    $bonk=1;
    return RC_OK;
}



#######################################
sub post_top
{
    my($request, $response)=@_;
    ok(($shutdown or (not $bonk and $request->is_error)),
            "all but shutdown requests should be errors");
}

#######################################
sub post_shutdown
{
    my($request, $response)=@_;
    ok($shutdown, "we are after shutdown");
}

#######################################
sub shutdown
{
    my ($request, $response) = @_;
    DEBUG and warn "SHUTDOWN";
    $poe_kernel->post($aliases->{httpd} => 'shutdown');
    $poe_kernel->post($aliases->{tcp} => 'shutdown');

    $shutdown=1;

    $response->code(RC_OK);
    $response->content_type('text/plain');
    $response->content("going to shutdown");
    return RC_OK;
}

